#include "waveshare_io_port.h"

uint8_t num = 0; // Variable to count events

#if EXAMPLE_ENABLE_PRINT_LCD_FPS
DRAM_ATTR int frame_count = 0; // Frame count for FPS calculation
int fps = 0; // Frames per second
DRAM_ATTR long start_time = 0; // Start time for FPS measurement

// Callback function for end of VSync
IRAM_ATTR bool onVsyncEndCallback(void *user_data)
{
    long frame_start_time = *(long *)user_data; // Get frame start time
    if (frame_start_time == 0) {
        (*(long *)user_data) = millis(); // Update frame start time
        return false; // Continue the loop
    }

    frame_count++; // Increment frame count
    if (frame_count >= EXAMPLE_LCD_FPS_COUNT_MAX) {
        fps = EXAMPLE_LCD_FPS_COUNT_MAX * 1000 / (millis() - frame_start_time); // Calculate FPS
        frame_count = 0; // Reset frame count
        (*(long *)user_data) = millis(); // Update frame start time
    }

    return false; // Continue the loop
}
#endif

// Function to test IO functionality
void waveshare_io_test(void)
{
    Serial.println("Initialize IO expander"); // Print initialization message
    /* Initialize IO expander */
    ESP_IOExpander_CH422G *expander = new ESP_IOExpander_CH422G((i2c_port_t)I2C_MASTER_NUM, ESP_IO_EXPANDER_I2C_CH422G_ADDRESS, I2C_MASTER_SCL_IO, I2C_MASTER_SDA_IO);
    // Create an instance of the IO expander
    expander->init(); // Initialize the expander
    expander->begin(); // Begin expander operation

    Serial.println("Set the OC pin to push-pull output mode.");
    expander->enableOC_PushPull();

    Serial.println("Set the IO0-7 pin to input mode.");
    expander->enableAllIO_Input();

    // Set output pins to default high
    expander->digitalWrite(DO0, 1); // Set DO0 high
    expander->digitalWrite(DO1, 1); // Set DO1 high

    Serial.println("IO test example start"); // Print start message
    while(1)
    {
        // Control output pins
        expander->digitalWrite(DO0, 1); // Set DO0 high
        expander->digitalWrite(DO1, 0); // Set DO1 low
        delay(1); // Delay for 1 millisecond
        if (expander->multiDigitalRead(DI0_mask | DI1_mask) == 0x01) // Check input state
            num++; // Increment num if DI0 is detected

        expander->digitalWrite(DO0, 0); // Set DO0 low
        expander->digitalWrite(DO1, 1); // Set DO1 high
        delay(1); // Delay for 1 millisecond
        if (expander->multiDigitalRead(DI0_mask | DI1_mask) == 0x20) // Check input state
            num++; // Increment num if DI1 is detected
        
        // Break the loop if num reaches 2
        if(num == 2)
            break; // Exit the loop
        else
            num = 3; // Reset num
        break; // Exit the loop
    }
}

bool colorFill(ESP_PanelLcd *lcd, uint16_t width, uint16_t height, uint16_t color)
{
  int bits_per_piexl = lcd->getColorBits();
  ESP_PANEL_CHECK_FALSE_RET(bits_per_piexl > 0, false, "Invalid color bits");

  int bytes_per_piexl = bits_per_piexl / 8;
  uint8_t *single_bar_buf = NULL;

  /* Malloc memory for a single color bar */
  try {
      single_bar_buf = new uint8_t[height * width * bytes_per_piexl];
  } catch (std::bad_alloc &e) {
      ESP_PANEL_CHECK_FALSE_RET(false, false, "Malloc color buffer failed");
  }

  bool ret = true;

    /* Draws full-screen colors */
    for (int j = 0; j < bits_per_piexl; j++) {
        for (int i = 0; i < height * width; i++) {
            for (int k = 0; k < bytes_per_piexl; k++) {
                single_bar_buf[i * bytes_per_piexl + k] = (color >> (k * 8)) & 0xff;
            }  
        }
        ret = lcd->drawBitmap(0, 0, width, height, single_bar_buf);
        if (ret != true) {
            ESP_LOGE(TAG, "Draw bitmap failed");
            goto end;
        }
    }

end:
    delete[] single_bar_buf;
    return ret;
}

// Function to initialize the LCD
void waveshare_lcd_init(void)
{
    Serial.println("Create RGB LCD bus"); // Print initialization message
    // Create an RGB panel bus
    ESP_PanelBus_RGB *panel_bus = new ESP_PanelBus_RGB(EXAMPLE_LCD_WIDTH, EXAMPLE_LCD_HEIGHT,
                                                       EXAMPLE_LCD_PIN_NUM_RGB_DATA0, EXAMPLE_LCD_PIN_NUM_RGB_DATA1,
                                                       EXAMPLE_LCD_PIN_NUM_RGB_DATA2, EXAMPLE_LCD_PIN_NUM_RGB_DATA3,
                                                       EXAMPLE_LCD_PIN_NUM_RGB_DATA4, EXAMPLE_LCD_PIN_NUM_RGB_DATA5,
                                                       EXAMPLE_LCD_PIN_NUM_RGB_DATA6, EXAMPLE_LCD_PIN_NUM_RGB_DATA7,
                                                       EXAMPLE_LCD_PIN_NUM_RGB_DATA8, EXAMPLE_LCD_PIN_NUM_RGB_DATA9,
                                                       EXAMPLE_LCD_PIN_NUM_RGB_DATA10, EXAMPLE_LCD_PIN_NUM_RGB_DATA11,
                                                       EXAMPLE_LCD_PIN_NUM_RGB_DATA12, EXAMPLE_LCD_PIN_NUM_RGB_DATA13,
                                                       EXAMPLE_LCD_PIN_NUM_RGB_DATA14, EXAMPLE_LCD_PIN_NUM_RGB_DATA15,
                                                       EXAMPLE_LCD_PIN_NUM_RGB_HSYNC, EXAMPLE_LCD_PIN_NUM_RGB_VSYNC,
                                                       EXAMPLE_LCD_PIN_NUM_RGB_PCLK, EXAMPLE_LCD_PIN_NUM_RGB_DE,
                                                       EXAMPLE_LCD_PIN_NUM_RGB_DISP);

    // Configure RGB timing frequency
    panel_bus->configRgbTimingFreqHz(EXAMPLE_LCD_RGB_TIMING_FREQ_HZ);
    // Configure RGB timing porch settings
    panel_bus->configRgbTimingPorch(EXAMPLE_LCD_RGB_TIMING_HPW, EXAMPLE_LCD_RGB_TIMING_HBP, EXAMPLE_LCD_RGB_TIMING_HFP,
                                    EXAMPLE_LCD_RGB_TIMING_VPW, EXAMPLE_LCD_RGB_TIMING_VBP, EXAMPLE_LCD_RGB_TIMING_VFP);
    panel_bus->configRgbBounceBufferSize(EXAMPLE_LCD_WIDTH * 10); // Set bounce buffer size
    panel_bus->configRgbFlagDispActiveLow(); // Set display active low
    panel_bus->begin(); // Begin panel bus operation

    Serial.println("Create LCD device"); // Print message
    // Create an instance of the LCD
    ESP_PanelLcd *lcd = new EXAMPLE_LCD_CLASS(EXAMPLE_LCD_NAME, panel_bus, EXAMPLE_LCD_COLOR_BITS, EXAMPLE_LCD_PIN_NUM_RST);
    lcd->init(); // Initialize the LCD
    lcd->reset(); // Reset the LCD
    lcd->begin(); // Begin LCD operation

#if EXAMPLE_ENABLE_PRINT_LCD_FPS
    lcd->attachRefreshFinishCallback(onVsyncEndCallback, (void *)&start_time); // Attach callback for VSync end
#endif
    Serial.println("Draws full-screen colors"); // Print message
    // Fill the LCD with a color based on the num variable
    if(num == 2)
        colorFill(lcd,EXAMPLE_LCD_WIDTH, EXAMPLE_LCD_HEIGHT, GREEN); // Fill with green
    else
        colorFill(lcd,EXAMPLE_LCD_WIDTH, EXAMPLE_LCD_HEIGHT, RED); // Fill with red
}
